@@ -26,6 +26,18 @@
 
 #include "igraph.h"
 
+#undef __BEGIN_DECLS
+#undef __END_DECLS
+#ifdef __cplusplus
+# define __BEGIN_DECLS extern "C" {
+# define __END_DECLS }
+#else
+# define __BEGIN_DECLS /* empty */
+# define __END_DECLS /* empty */
+#endif
+
+__BEGIN_DECLS
+
 /* -------------------------------------------------- */
 /* Attributes                                         */
 /* -------------------------------------------------- */
@@ -78,6 +90,12 @@ typedef enum { IGRAPH_ATTRIBUTE_GRAPH=0,
  * \member delete_edges Called when edges were deleted. The edge
  *    recoding vector is supplied, in the same form as for the \c
  *    delete_vertices function.
+ * \member permute_edges Typically called when a new graph is created and 
+ *    some of the new edges should carry the attributes of some of the
+ *    old edges. The idx vector shows the mapping between the old edges and 
+ *    the new ones. Its length is the same as the number of edges in the new 
+ *    graph, and for each edge it gives the id of the old edge (the edge in
+ *    the old graph).
  */
 
 typedef struct igraph_attribute_table_t {
@@ -90,6 +108,7 @@ typedef struct igraph_attribute_table_t {
   int (*add_edges)(igraph_t *graph, const igraph_vector_t *edges, 
 		   igraph_vector_ptr_t *attr);
   void (*delete_edges)(igraph_t *graph, const igraph_vector_t *idx);
+  int (*permute_edges)(igraph_t *graph, const igraph_vector_t *idx);
 
   int (*get_info)(const igraph_t *graph,
 		  igraph_strvector_t *gnames, igraph_vector_t *gtypes,
@@ -130,8 +149,8 @@ igraph_i_set_attribute_table(igraph_attribute_table_t * table);
         do {if ((graph)->attr) igraph_i_attribute_delete_vertices((graph),(eidx),(vidx));} while(0)
 #define IGRAPH_I_ATTRIBUTE_COPY(to,from) do { \
         int igraph_i_ret=0; \
-        if (from->attr) { \
-          IGRAPH_CHECK(igraph_i_ret=igraph_i_attribute_copy(to, from)); \
+        if ((from)->attr) { \
+          IGRAPH_CHECK(igraph_i_ret=igraph_i_attribute_copy((to), (from))); \
         } \
         if (igraph_i_ret != 0) { \
           IGRAPH_ERROR("", igraph_i_ret); \
@@ -149,6 +168,8 @@ int igraph_i_attribute_add_edges(igraph_t *graph,
 				 const igraph_vector_t *edges, void *attr);
 void igraph_i_attribute_delete_edges(igraph_t *graph, 
 				     const igraph_vector_t *idx);
+int igraph_i_attribute_permute_edges(igraph_t *graph, 
+				     const igraph_vector_t *idx);
 
 int igraph_i_attribute_get_info(const igraph_t *graph,
 				igraph_strvector_t *gnames, 
@@ -188,4 +209,6 @@ int igraph_i_attribute_get_string_edge_attr(const igraph_t *graph,
 					    igraph_es_t es,
 					    igraph_strvector_t *value);
 
+__END_DECLS
+
 #endif
