@@ -25,10 +25,12 @@
 #include "config.h"
 
 #include <ctype.h>		/* isspace */
+#include <math.h>               /* isnan */
 #include <string.h>
 #include "memory.h"
+#include <stdarg.h> 		/* va_start & co */
 
-#ifdef HAVE_LIBXML
+#if HAVE_LIBXML == 1
 #include <libxml/encoding.h>
 #include <libxml/parser.h>
 
@@ -62,25 +64,30 @@ struct igraph_i_graphml_parser_state {
   unsigned int prev_state;
   unsigned int unknown_depth;
   int index;
-  igraph_bool_t successful, edges_directed;
+  igraph_bool_t successful, edges_directed, destroyed;
   igraph_trie_t v_names;
   igraph_vector_ptr_t v_attrs;
   igraph_trie_t e_names;
   igraph_vector_ptr_t e_attrs;
   xmlChar *data_key;
   igraph_attribute_elemtype_t data_type;
+  char *error_message;
 };
 
 void igraph_i_graphml_destroy_state(struct igraph_i_graphml_parser_state* state) {
   long int i;
 
+  if (state->destroyed) return;
+  state->destroyed=1;
+
   /* this is the easy part */
   igraph_trie_destroy(&state->node_trie);
   igraph_strvector_destroy(&state->edgeids);
   igraph_trie_destroy(&state->v_names);
   igraph_trie_destroy(&state->e_names);
   igraph_vector_destroy(&state->edgelist);
-
+   
+  if (state->error_message) { free(state->error_message); }
   if (state->data_key) { free(state->data_key); }
   
   for (i=0; i<igraph_vector_ptr_size(&state->v_attrs); i++) {
@@ -96,6 +103,8 @@ void igraph_i_graphml_destroy_state(struct igraph_i_graphml_parser_state* state)
 	Free(rec->record.value);
       }
     }
+    if (rec->id != 0) Free(rec->id);
+    if (rec->record.name != 0) Free(rec->record.name);
     Free(rec);
   }	 
 
@@ -112,19 +121,32 @@ void igraph_i_graphml_destroy_state(struct igraph_i_graphml_parser_state* state)
 	Free(rec->record.value);
       }
     }
+    if (rec->id != 0) Free(rec->id);
+    if (rec->record.name != 0) Free(rec->record.name);
     Free(rec);
   }
 
+  igraph_vector_ptr_destroy(&state->v_attrs);
+  igraph_vector_ptr_destroy(&state->e_attrs);
+  
   IGRAPH_FINALLY_CLEAN(1);
 }
 
 void igraph_i_graphml_sax_handler_error(void *state0, const char* msg, ...) {
   struct igraph_i_graphml_parser_state *state=
     (struct igraph_i_graphml_parser_state*)state0;
+  va_list ap;
+  
+  va_start(ap, msg);
+  
+  if (state->error_message == 0)
+    state->error_message=Calloc(4096, char);
+   
   state->successful=0;
   state->st=ERROR;
-  igraph_i_graphml_destroy_state(state); /* Is this correct???? */
-  /* TODO: use the message */
+  vsnprintf(state->error_message, 4096, msg, ap);
+   
+  va_end(ap);
 }
 
 xmlEntityPtr igraph_i_graphml_sax_handler_get_entity(void *state0,
@@ -148,40 +170,56 @@ void igraph_i_graphml_sax_handler_start_document(void *state0) {
   state->st=START;
   state->successful=1;
   state->edges_directed=0;
+  state->destroyed=0;
   state->data_key=0;
+  state->error_message=0;
   ret=igraph_vector_ptr_init(&state->v_attrs, 0);
   if (ret) {
     igraph_error("Cannot parse GraphML file", __FILE__, __LINE__, ret);
+    igraph_i_graphml_sax_handler_error(state, "Cannot parse GraphML file");
+    return;
   }
   IGRAPH_FINALLY(igraph_vector_ptr_destroy, &state->v_attrs);
   ret=igraph_vector_ptr_init(&state->e_attrs, 0);
   if (ret) {
     igraph_error("Cannot parse GraphML file", __FILE__, __LINE__, ret);
+    igraph_i_graphml_sax_handler_error(state, "Cannot parse GraphML file");
+    return;
   }
   IGRAPH_FINALLY(igraph_vector_ptr_destroy, &state->e_attrs);
   ret=igraph_vector_init(&state->edgelist, 0);
   if (ret) {
     igraph_error("Cannot parse GraphML file", __FILE__, __LINE__, ret);
+    igraph_i_graphml_sax_handler_error(state, "Cannot parse GraphML file");
+    return;
   }
   IGRAPH_FINALLY(igraph_vector_destroy, &state->edgelist);
   ret=igraph_trie_init(&state->node_trie, 1);
   if (ret) {
     igraph_error("Cannot parse GraphML file", __FILE__, __LINE__, ret);
+    igraph_i_graphml_sax_handler_error(state, "Cannot parse GraphML file");
+    return;
   }
   IGRAPH_FINALLY(igraph_trie_destroy, &state->node_trie);
   ret=igraph_strvector_init(&state->edgeids, 0);
   if (ret) {
     igraph_error("Cannot parse GraphML file", __FILE__, __LINE__, ret);
+    igraph_i_graphml_sax_handler_error(state, "Cannot parse GraphML file");
+    return;
   }
   IGRAPH_FINALLY(igraph_strvector_destroy, &state->edgeids);
   ret=igraph_trie_init(&state->v_names, 0);
   if (ret) {
     igraph_error("Cannot parse GraphML file", __FILE__, __LINE__, ret);
+    igraph_i_graphml_sax_handler_error(state, "Cannot parse GraphML file");
+    return;
   }
   IGRAPH_FINALLY(igraph_trie_destroy, &state->v_names);
   ret=igraph_trie_init(&state->e_names, 0);
   if (ret) {
     igraph_error("Cannot parse GraphML file", __FILE__, __LINE__, ret);
+    igraph_i_graphml_sax_handler_error(state, "Cannot parse GraphML file");
+    return;
   }
   IGRAPH_FINALLY(igraph_trie_destroy, &state->e_names);
   
@@ -197,6 +235,8 @@ void igraph_i_graphml_sax_handler_end_document(void *state0) {
   igraph_i_attribute_record_t idrec, eidrec;
   const char *idstr="id";
 
+  if (!state->successful) return;
+
   if (state->index<0) {
 
     igraph_vector_ptr_t vattr, eattr;
@@ -205,6 +245,8 @@ void igraph_i_graphml_sax_handler_end_document(void *state0) {
 			     igraph_vector_ptr_size(&state->v_attrs)+1);
     if (r) {
       igraph_error("Cannot parse GraphML file", __FILE__, __LINE__, r);
+      igraph_i_graphml_sax_handler_error(state, "Cannot parse GraphML file");
+      return;
     }
     IGRAPH_FINALLY(igraph_vector_ptr_destroy, &vattr);
     if (igraph_strvector_size(&state->edgeids) != 0) {
@@ -213,6 +255,8 @@ void igraph_i_graphml_sax_handler_end_document(void *state0) {
     r=igraph_vector_ptr_init(&eattr, esize);
     if (r) {
       igraph_error("Cannot parse GraphML file", __FILE__, __LINE__, r);
+      igraph_i_graphml_sax_handler_error(state, "Cannot parse GraphML file");
+      return;
     }
     IGRAPH_FINALLY(igraph_vector_ptr_destroy, &eattr);
 
@@ -289,6 +333,7 @@ void igraph_i_graphml_sax_handler_end_document(void *state0) {
     igraph_vector_ptr_destroy(&vattr);
     igraph_vector_ptr_destroy(&eattr);
     IGRAPH_FINALLY_CLEAN(2);
+     
   }
 
   igraph_i_graphml_destroy_state(state);
@@ -300,15 +345,20 @@ void igraph_i_graphml_sax_handler_end_document(void *state0) {
 void igraph_i_graphml_add_attribute_key(const xmlChar** attrs, 
 					struct igraph_i_graphml_parser_state *state) {
   xmlChar **it;
-  igraph_trie_t *trie;
-  igraph_vector_ptr_t *ptrvector;
+  igraph_trie_t *trie=0;
+  igraph_vector_ptr_t *ptrvector=0;
   long int id;
   int ret;
   igraph_i_graphml_attribute_record_t *rec=
     Calloc(1, igraph_i_graphml_attribute_record_t);
-  if (rec==0) { 
+
+  if (!state->successful) return;
+   
+   if (rec==0) { 
     igraph_error("Cannot parse GraphML file", __FILE__, __LINE__, 
 		 IGRAPH_ENOMEM);
+    igraph_i_graphml_sax_handler_error(state, "Cannot parse GraphML file");
+    return;
   }
   IGRAPH_FINALLY(igraph_free, rec);
   for (it=(xmlChar**)attrs; *it; it+=2) {
@@ -320,7 +370,7 @@ void igraph_i_graphml_add_attribute_key(const xmlChar** attrs,
       rec->record.name=strdup(name);
     } else if (xmlStrEqual(*it, toXmlChar("attr.type"))) {
       if (xmlStrEqual(*(it+1), (xmlChar*)"boolean")) { 
-	rec->type==I_GRAPHML_BOOLEAN;
+	rec->type=I_GRAPHML_BOOLEAN;
 	rec->record.type=IGRAPH_ATTRIBUTE_NUMERIC;	    
       } else if (xmlStrEqual(*(it+1), toXmlChar("string"))) {
 	rec->type=I_GRAPHML_STRING;
@@ -340,12 +390,16 @@ void igraph_i_graphml_add_attribute_key(const xmlChar** attrs,
       } else {
 	igraph_error("Cannot parse GraphML file, unknown attribute type", 
 		     __FILE__, __LINE__, IGRAPH_PARSEERROR);
+        igraph_i_graphml_sax_handler_error(state, "Cannot parse GraphML file, unknown attribute type");
+        return;
       }
     } else if (xmlStrEqual(*it, toXmlChar("for"))) {
       /* graph, vertex or edge attribute? */
       if (xmlStrEqual(*(it+1), toXmlChar("graph"))) { 
 	igraph_error("Cannot parse GraphML file, graph attributes not implemented",
 		     __FILE__, __LINE__, IGRAPH_UNIMPLEMENTED);	
+        igraph_i_graphml_sax_handler_error(state, "Cannot parse GraphML file, graph attributes not implemented");
+        return;
       } else if (xmlStrEqual(*(it+1), toXmlChar("node"))) {
 	trie=&state->v_names;
 	ptrvector=&state->v_attrs;
@@ -355,19 +409,31 @@ void igraph_i_graphml_add_attribute_key(const xmlChar** attrs,
       } else {
 	igraph_error("Cannot parse GraphML file, unknown attribute type",
 		     __FILE__, __LINE__, IGRAPH_PARSEERROR);
+        igraph_i_graphml_sax_handler_error(state, "Cannot parse GraphML file, unknown attribute type");
+        return;
       }
     }
   }
 
+  if (trie == 0 && state->successful) {
+    igraph_error("Cannot parse GraphML file, missing 'for' attribute", __FILE__, __LINE__, IGRAPH_PARSEERROR);
+    igraph_i_graphml_sax_handler_error(state, "Cannot parse GraphML file, missing 'for' attribute");
+    return;
+  }
+	
   /* add to trie, attribues */
   igraph_trie_get(trie, rec->id, &id);
   if (id != igraph_trie_size(trie)-1) {
     igraph_error("Cannot parse GraphML file, duplicate attribute", 
 		 __FILE__, __LINE__, IGRAPH_PARSEERROR);
+    igraph_i_graphml_sax_handler_error(state, "Cannot parse GraphML file, duplicate attribute");
+    return;
   }
   ret=igraph_vector_ptr_push_back(ptrvector, rec);
   if (ret) {
     igraph_error("Cannot read GraphML file", __FILE__, __LINE__, ret);
+    igraph_i_graphml_sax_handler_error(state, "Cannot read GraphML file");
+    return;
   }
 
   /* create the attribute values */
@@ -379,6 +445,8 @@ void igraph_i_graphml_add_attribute_key(const xmlChar** attrs,
     if (vec==0) {
       igraph_error("Cannot parse GraphML file", __FILE__, __LINE__,
 		   IGRAPH_ENOMEM);
+      igraph_i_graphml_sax_handler_error(state, "Cannot parse GraphML file");
+      return;
     }
     rec->record.value=vec;
     igraph_vector_init(vec, 0);    
@@ -386,8 +454,10 @@ void igraph_i_graphml_add_attribute_key(const xmlChar** attrs,
   case IGRAPH_ATTRIBUTE_STRING:
     strvec=Calloc(1, igraph_strvector_t);
     if (strvec==0) {
-      igraph_error("Cannot parge GraphML file", __FILE__, __LINE__, 
+      igraph_error("Cannot parse GraphML file", __FILE__, __LINE__, 
 		   IGRAPH_ENOMEM);
+      igraph_i_graphml_sax_handler_error(state, "Cannot parse GraphML file");
+      return;
     }
     rec->record.value=strvec;
     igraph_strvector_init(strvec, 0);
@@ -421,18 +491,22 @@ void igraph_i_graphml_attribute_data_add(struct igraph_i_graphml_parser_state *s
   char *chardata;
   const xmlChar *end=xmlStrchr(data, (xmlChar) '<');
   igraph_attribute_elemtype_t type=state->data_type;
-  igraph_trie_t *trie;
-  igraph_vector_ptr_t *ptrvector;
+  igraph_trie_t *trie=0;
+  igraph_vector_ptr_t *ptrvector=0;
   igraph_i_graphml_attribute_record_t *graphmlrec;
   igraph_i_attribute_record_t *rec;
   long int recid;
-  long int id;
+  long int id=0;
   int ret;
 
+  if (!state->successful) return;
+
   chardata=Calloc( (end-data)+1, char);  
   if (chardata==0) {
     igraph_error("Cannot parse GraphML file", __FILE__, __LINE__, 
 		 IGRAPH_ENOMEM);
+    igraph_i_graphml_sax_handler_error(state, "Cannot parse GraphML file");
+    return;
   }
   memcpy(chardata, data, (end-data)*sizeof(char));
   chardata[(end-data)]='\0';
@@ -448,7 +522,7 @@ void igraph_i_graphml_attribute_data_add(struct igraph_i_graphml_parser_state *s
     ptrvector=&state->e_attrs;
     id=igraph_vector_size(&state->edgelist)/2-1; /* hack */
     break;
-  defaults:
+  default:
     /* impossible */
     break;
   }
@@ -469,6 +543,8 @@ void igraph_i_graphml_attribute_data_add(struct igraph_i_graphml_parser_state *s
       ret=igraph_vector_resize(vec, id+1);
       if (ret) {
 	igraph_error("Cannot parse GraphML file", __FILE__, __LINE__, ret);
+        igraph_i_graphml_sax_handler_error(state, "Cannot parse GraphML file");
+        return;
       }
       for (i=s; i<id; i++) {
 	VECTOR(*vec)[i]=0.0/0.0;
@@ -484,6 +560,8 @@ void igraph_i_graphml_attribute_data_add(struct igraph_i_graphml_parser_state *s
       ret=igraph_strvector_resize(strvec, id+1);
       if (ret) {
 	igraph_error("Cannot parse GraphML file", __FILE__, __LINE__, ret);
+        igraph_i_graphml_sax_handler_error(state, "Cannot parse GraphML file");
+        return;
       }
       for (i=s;i<id;i++) {
 	igraph_strvector_set(strvec, i, "");
@@ -492,6 +570,8 @@ void igraph_i_graphml_attribute_data_add(struct igraph_i_graphml_parser_state *s
     ret=igraph_strvector_set(strvec, id, chardata);
     if (ret) {
       igraph_error("Cannot parse GraphML file", __FILE__, __LINE__, ret);
+      igraph_i_graphml_sax_handler_error(state, "Cannot parse GraphML file");
+      return;
     }
     break;
   default:
@@ -700,6 +780,38 @@ static xmlSAXHandler igraph_i_graphml_sax_handler={
 
 #endif
 
+int igraph_i_xml_escape(char* src, char** dest) {
+  long int destlen=0;
+  char *s, *d;
+  for (s=src; *s; s++, destlen++) {
+    if (*s == '&') destlen += 4;
+    else if (*s == '<') destlen += 3;
+    else if (*s == '>') destlen += 3;
+    else if (*s == '"') destlen += 5;
+    else if (*s == '\'') destlen += 5;
+  }
+  *dest=Calloc(destlen+1, char);
+  if (!*dest) IGRAPH_ERROR("Not enough memory", IGRAPH_ENOMEM);
+  for (s=src, d=*dest; *s; s++, d++) {
+    switch (*s) {
+    case '&':
+      strcpy(d, "&amp;"); d+=4; break;
+    case '<':
+      strcpy(d, "&lt;"); d+=3; break;
+    case '>':
+      strcpy(d, "&gt;"); d+=3; break;
+    case '"':
+      strcpy(d, "&quot;"); d+=5; break;
+    case '\'':
+      strcpy(d, "&apos;"); d+=5; break;
+    default:
+      *d = *s;
+    }
+  }
+  *d=0;
+  return 0;
+}
+
 /**
  * \ingroup loadsave
  * \function igraph_read_graph_graphml
@@ -727,7 +839,7 @@ static xmlSAXHandler igraph_i_graphml_sax_handler={
 int igraph_read_graph_graphml(igraph_t *graph, FILE *instream,
 			      int index) {
 
-#ifdef HAVE_LIBXML
+#if HAVE_LIBXML == 1
   xmlParserCtxtPtr ctxt;
   struct igraph_i_graphml_parser_state state;
   int res;
@@ -755,8 +867,12 @@ int igraph_read_graph_graphml(igraph_t *graph, FILE *instream,
   
   // Free the context
   xmlFreeParserCtxt(ctxt);
-  if (!state.successful)
-    IGRAPH_ERROR("Malformed GraphML file", IGRAPH_PARSEERROR);
+  if (!state.successful) {
+    if (state.error_message != 0)
+      IGRAPH_ERROR(state.error_message, IGRAPH_PARSEERROR);
+    else
+      IGRAPH_ERROR("Malformed GraphML file", IGRAPH_PARSEERROR);
+  }
   if (state.index>=0)
     IGRAPH_ERROR("Graph index was too large", IGRAPH_EINVAL);
   
@@ -831,41 +947,47 @@ int igraph_write_graph_graphml(const igraph_t *graph, FILE *outstream) {
   
   /* graph attributes */
   for (i=0; i<igraph_vector_size(&gtypes); i++) {
-    char *name;
+    char *name, *name_escaped;
     igraph_strvector_get(&gnames, i, &name);
+    IGRAPH_CHECK(igraph_i_xml_escape(name, &name_escaped));
     if (VECTOR(gtypes)[i] == IGRAPH_ATTRIBUTE_STRING) {
-      ret=fprintf(outstream, "  <key id=\"%s\" for=\"graph\" attr.name=\"%s\" attr.type=\"string\"/>\n", name, name);
+      ret=fprintf(outstream, "  <key id=\"%s\" for=\"graph\" attr.name=\"%s\" attr.type=\"string\"/>\n", name_escaped, name_escaped);
       if (ret<0) IGRAPH_ERROR("Write failed", IGRAPH_EFILE);      
     } else if (VECTOR(gtypes)[i] == IGRAPH_ATTRIBUTE_NUMERIC) {
-      ret=fprintf(outstream, "  <key id=\"%s\" for=\"graph\" attr.name=\"%s\" attr.type=\"double\"/>\n", name, name);
+      ret=fprintf(outstream, "  <key id=\"%s\" for=\"graph\" attr.name=\"%s\" attr.type=\"double\"/>\n", name_escaped, name_escaped);
       if (ret<0) IGRAPH_ERROR("Write failed", IGRAPH_EFILE);            
     }
+    Free(name_escaped);
   }
 
   /* vertex attributes */
   for (i=0; i<igraph_vector_size(&vtypes); i++) {
-    char *name;
+    char *name, *name_escaped;
     igraph_strvector_get(&vnames, i, &name);
+    IGRAPH_CHECK(igraph_i_xml_escape(name, &name_escaped));
     if (VECTOR(vtypes)[i] == IGRAPH_ATTRIBUTE_STRING) {
-      ret=fprintf(outstream, "  <key id=\"%s\" for=\"node\" attr.name=\"%s\" attr.type=\"string\"/>\n", name, name);
+      ret=fprintf(outstream, "  <key id=\"%s\" for=\"node\" attr.name=\"%s\" attr.type=\"string\"/>\n", name_escaped, name_escaped);
       if (ret<0) IGRAPH_ERROR("Write failed", IGRAPH_EFILE);      
     } else if (VECTOR(vtypes)[i] == IGRAPH_ATTRIBUTE_NUMERIC) {
-      ret=fprintf(outstream, "  <key id=\"%s\" for=\"node\" attr.name=\"%s\" attr.type=\"double\"/>\n", name, name);
+      ret=fprintf(outstream, "  <key id=\"%s\" for=\"node\" attr.name=\"%s\" attr.type=\"double\"/>\n", name_escaped, name_escaped);
       if (ret<0) IGRAPH_ERROR("Write failed", IGRAPH_EFILE);            
     }
+    Free(name_escaped);
   }
 
   /* edge attributes */
   for (i=0; i<igraph_vector_size(&etypes); i++) {
-    char *name;
+    char *name, *name_escaped;
     igraph_strvector_get(&enames, i, &name);
+    IGRAPH_CHECK(igraph_i_xml_escape(name, &name_escaped));
     if (VECTOR(etypes)[i] == IGRAPH_ATTRIBUTE_STRING) {
-      ret=fprintf(outstream, "  <key id=\"%s\" for=\"edge\" attr.name=\"%s\" attr.type=\"string\"/>\n", name, name);
+      ret=fprintf(outstream, "  <key id=\"%s\" for=\"edge\" attr.name=\"%s\" attr.type=\"string\"/>\n", name_escaped, name_escaped);
       if (ret<0) IGRAPH_ERROR("Write failed", IGRAPH_EFILE);
     } else if (VECTOR(etypes)[i] == IGRAPH_ATTRIBUTE_NUMERIC) {
-      ret=fprintf(outstream, "  <key id=\"%s\" for=\"edge\" attr.name=\"%s\" attr.type=\"double\"/>\n", name, name);
+      ret=fprintf(outstream, "  <key id=\"%s\" for=\"edge\" attr.name=\"%s\" attr.type=\"double\"/>\n", name_escaped, name_escaped);
       if (ret<0) IGRAPH_ERROR("Write failed", IGRAPH_EFILE);
     }
+    Free(name_escaped);
   }
 
   ret=fprintf(outstream, "  <graph id=\"G\" edgedefault=\"%s\">\n", (igraph_is_directed(graph)?"directed":"undirected"));
@@ -874,23 +996,28 @@ int igraph_write_graph_graphml(const igraph_t *graph, FILE *outstream) {
   /* Write the graph atributes before anything else */
   
   for (i=0; i<igraph_vector_size(&gtypes); i++) {
-    char *name;
+    char *name, *name_escaped;
     if (VECTOR(gtypes)[i] == IGRAPH_ATTRIBUTE_NUMERIC) {
       igraph_strvector_get(&gnames, i, &name);
-      ret=fprintf(outstream, "  <data key=\"%s\">", name);
-      if (ret<0) IGRAPH_ERROR("Write failed", IGRAPH_EFILE);
       igraph_i_attribute_get_numeric_graph_attr(graph, name, &numv);
-      ret=fprintf(outstream, "%g", VECTOR(numv)[0]);
-      if (ret<0) IGRAPH_ERROR("Write failed", IGRAPH_EFILE);
-      ret=fprintf(outstream, "</data>\n");
-	if (ret<0) IGRAPH_ERROR("Write failed", IGRAPH_EFILE);      
+      if (!isnan(VECTOR(numv)[0])) {
+	IGRAPH_CHECK(igraph_i_xml_escape(name, &name_escaped));
+	ret=fprintf(outstream, "    <data key=\"%s\">%g</data>\n",
+		    name_escaped, VECTOR(numv)[0]);
+	Free(name_escaped);
+	if (ret<0) IGRAPH_ERROR("Write failed", IGRAPH_EFILE);
+      }
     } else if (VECTOR(gtypes)[i] == IGRAPH_ATTRIBUTE_STRING) {
-      char *s;
+      char *s, *s_escaped;
       igraph_strvector_get(&gnames, i, &name);
-      ret=fprintf(outstream, "  <data key=\"%s\">", name);
+      IGRAPH_CHECK(igraph_i_xml_escape(name, &name_escaped));
+      ret=fprintf(outstream, "    <data key=\"%s\">", name_escaped);
+      Free(name_escaped);
       igraph_i_attribute_get_string_graph_attr(graph, name, &strv);
       igraph_strvector_get(&strv, 0, &s);
-      ret=fprintf(outstream, "%s", s);
+      IGRAPH_CHECK(igraph_i_xml_escape(s, &s_escaped));
+      ret=fprintf(outstream, "%s", s_escaped);
+      Free(s_escaped);
       if (ret<0) IGRAPH_ERROR("Write failed", IGRAPH_EFILE);
       ret=fprintf(outstream, "</data>\n");
       if (ret<0) IGRAPH_ERROR("Write failed", IGRAPH_EFILE);
@@ -900,36 +1027,42 @@ int igraph_write_graph_graphml(const igraph_t *graph, FILE *outstream) {
   /* Let's dump the nodes first */
   vc=igraph_vcount(graph);
   for (l=0; l<vc; l++) {
-    char *name;
+    char *name, *name_escaped;
     ret=fprintf(outstream, "    <node id=\"n%ld\">\n", (long)l);
+
     if (ret<0) IGRAPH_ERROR("Write failed", IGRAPH_EFILE);
     
     for (i=0; i<igraph_vector_size(&vtypes); i++) {
       if (VECTOR(vtypes)[i] == IGRAPH_ATTRIBUTE_NUMERIC) {
 	igraph_strvector_get(&vnames, i, &name);
-	ret=fprintf(outstream, "      <data key=\"%s\">", name);
-	if (ret<0) IGRAPH_ERROR("Write failed", IGRAPH_EFILE);
 	igraph_i_attribute_get_numeric_vertex_attr(graph, name,
 						   igraph_vss_1(l), &numv);
-	ret=fprintf(outstream, "%g", VECTOR(numv)[0]);
-	if (ret<0) IGRAPH_ERROR("Write failed", IGRAPH_EFILE);
-	ret=fprintf(outstream, "</data>\n");
-	if (ret<0) IGRAPH_ERROR("Write failed", IGRAPH_EFILE);      
+	if (!isnan(VECTOR(numv)[0])) {
+	  IGRAPH_CHECK(igraph_i_xml_escape(name, &name_escaped));
+	  ret=fprintf(outstream, "      <data key=\"%s\">%g</data>\n",
+		      name_escaped, VECTOR(numv)[0]);
+	  Free(name_escaped);
+	  if (ret<0) IGRAPH_ERROR("Write failed", IGRAPH_EFILE);
+	}
       } else if (VECTOR(vtypes)[i] == IGRAPH_ATTRIBUTE_STRING) {
-	char *s;
+	char *s, *s_escaped;
 	igraph_strvector_get(&vnames, i, &name);
-	ret=fprintf(outstream, "      <data key=\"%s\">", name);
+	IGRAPH_CHECK(igraph_i_xml_escape(name, &name_escaped));
+	ret=fprintf(outstream, "      <data key=\"%s\">", name_escaped);
+	Free(name_escaped);
 	igraph_i_attribute_get_string_vertex_attr(graph, name,
 						  igraph_vss_1(l), &strv);
 	igraph_strvector_get(&strv, 0, &s);
-	ret=fprintf(outstream, "%s", s);
+	IGRAPH_CHECK(igraph_i_xml_escape(s, &s_escaped));
+	ret=fprintf(outstream, "%s", s_escaped);
+	Free(s_escaped);
 	if (ret<0) IGRAPH_ERROR("Write failed", IGRAPH_EFILE);
 	ret=fprintf(outstream, "</data>\n");
 	if (ret<0) IGRAPH_ERROR("Write failed", IGRAPH_EFILE);
       }
     }
 
-    ret=fprintf(outstream, "    </node>\n", (long)l);
+    ret=fprintf(outstream, "    </node>\n");
     if (ret<0) IGRAPH_ERROR("Write failed", IGRAPH_EFILE);    
   }
   
@@ -938,7 +1071,7 @@ int igraph_write_graph_graphml(const igraph_t *graph, FILE *outstream) {
   IGRAPH_FINALLY(igraph_eit_destroy, &it);
   while (!IGRAPH_EIT_END(it)) {
     igraph_integer_t from, to;
-    char *name;
+    char *name, *name_escaped;
     long int edge=IGRAPH_EIT_GET(it);
     igraph_edge(graph, edge, &from, &to);
     ret=fprintf(outstream, "    <edge source=\"n%ld\" target=\"n%ld\">\n", 
@@ -948,22 +1081,28 @@ int igraph_write_graph_graphml(const igraph_t *graph, FILE *outstream) {
     for (i=0; i<igraph_vector_size(&etypes); i++) {
       if (VECTOR(etypes)[i] == IGRAPH_ATTRIBUTE_NUMERIC) {
 	igraph_strvector_get(&enames, i, &name);
-	ret=fprintf(outstream, "      <data key=\"%s\">", name);
-	if (ret<0) IGRAPH_ERROR("Write failed", IGRAPH_EFILE);
 	igraph_i_attribute_get_numeric_edge_attr(graph, name,
 						 igraph_ess_1(edge), &numv);
-	ret=fprintf(outstream, "%g", VECTOR(numv)[0]);
-	if (ret<0) IGRAPH_ERROR("Write failed", IGRAPH_EFILE);
-	ret=fprintf(outstream, "</data>\n");
+	if (!isnan(VECTOR(numv)[0])) {
+	  IGRAPH_CHECK(igraph_i_xml_escape(name, &name_escaped));
+	  ret=fprintf(outstream, "      <data key=\"%s\">%g</data>\n",
+		      name_escaped, VECTOR(numv)[0]);
+	  Free(name_escaped);
+	  if (ret<0) IGRAPH_ERROR("Write failed", IGRAPH_EFILE);
+	}
 	if (ret<0) IGRAPH_ERROR("Write failed", IGRAPH_EFILE);      
       } else if (VECTOR(etypes)[i] == IGRAPH_ATTRIBUTE_STRING) {
-	char *s;
+	char *s, *s_escaped;
 	igraph_strvector_get(&enames, i, &name);
-	ret=fprintf(outstream, "      <data key=\"%s\">", name);
+	IGRAPH_CHECK(igraph_i_xml_escape(name, &name_escaped));
+	ret=fprintf(outstream, "      <data key=\"%s\">", name_escaped);
+	Free(name_escaped);
 	igraph_i_attribute_get_string_edge_attr(graph, name,
 						igraph_ess_1(edge), &strv);
 	igraph_strvector_get(&strv, 0, &s);
-	ret=fprintf(outstream, "%s", s);
+	IGRAPH_CHECK(igraph_i_xml_escape(s, &s_escaped));
+	ret=fprintf(outstream, "%s", s_escaped);
+	Free(s_escaped);
 	if (ret<0) IGRAPH_ERROR("Write failed", IGRAPH_EFILE);
 	ret=fprintf(outstream, "</data>\n");
 	if (ret<0) IGRAPH_ERROR("Write failed", IGRAPH_EFILE);
