@@ -138,7 +138,7 @@ int igraph_barabasi_game(igraph_t *graph, igraph_integer_t n, igraph_integer_t m
   RNG_END();
 
   Free(bag);
-  IGRAPH_CHECK(igraph_create(graph, &edges, 0, directed));
+  IGRAPH_CHECK(igraph_create(graph, &edges, n, directed));
   igraph_vector_destroy(&edges);
   IGRAPH_FINALLY_CLEAN(2);
 
@@ -201,18 +201,21 @@ int igraph_erdos_renyi_game_gnp(igraph_t *graph, igraph_integer_t n, igraph_real
 
     if (directed && loops) {
       for (i=0; i<igraph_vector_size(&s); i++) {
-	igraph_vector_push_back(&edges, ((long int)(VECTOR(s)[i]-1))/no_of_nodes);
-	igraph_vector_push_back(&edges, ((long int)(VECTOR(s)[i]-1))%no_of_nodes);
+	igraph_real_t to=fmod(VECTOR(s)[i], no_of_nodes);
+	igraph_real_t from=(VECTOR(s)[i]-to)/no_of_nodes;
+	igraph_vector_push_back(&edges, from);
+	igraph_vector_push_back(&edges, to);
       }
     } else if (directed && !loops) {
       for (i=0; i<igraph_vector_size(&s); i++) {
-	long int from=((long int)(VECTOR(s)[i]-1))/(no_of_nodes-1);
-	long int to=((long int)VECTOR(s)[i])%(no_of_nodes-1);
+	igraph_real_t from=
+	  (VECTOR(s)[i]-1 - fmod(VECTOR(s)[i]-1, no_of_nodes-1))/(no_of_nodes-1);
+	igraph_real_t to=fmod(VECTOR(s)[i], no_of_nodes-1);
 	if (from==to) {
 	  to=no_of_nodes-1;
 	}
 	igraph_vector_push_back(&edges, from);
-	igraph_vector_push_back(&edges, to);
+	igraph_vector_push_back(&edges, to);	
       }
     } else if (!directed && loops) {
       for (i=0; i<igraph_vector_size(&s); i++) {
@@ -658,11 +661,13 @@ int igraph_callaway_traits_game (igraph_t *graph, igraph_integer_t nodes,
   igraph_vector_t edges;
   igraph_vector_t cumdist;
   igraph_real_t maxcum;
+  igraph_vector_t nodetypes;
 
   /* TODO: parameter checks */
 
   IGRAPH_VECTOR_INIT_FINALLY(&edges, 0);
   IGRAPH_VECTOR_INIT_FINALLY(&cumdist, types+1);
+  IGRAPH_VECTOR_INIT_FINALLY(&nodetypes, nodes);
   
   VECTOR(cumdist)[0]=0;
   for (i=0; i<types; i++) {
@@ -672,14 +677,19 @@ int igraph_callaway_traits_game (igraph_t *graph, igraph_integer_t nodes,
 
   RNG_BEGIN();
 
+  for (i=0; i<nodes; i++) {
+    igraph_real_t uni=RNG_UNIF(0, maxcum);
+    long int type;
+    igraph_vector_binsearch(&cumdist, uni, &type);
+    VECTOR(nodetypes)[i]=type;
+  }    
+
   for (i=1; i<nodes; i++) {
     for (j=0; j<edges_per_step; j++) {
       long int node1=RNG_INTEGER(0, i);
       long int node2=RNG_INTEGER(0, i);
-      igraph_real_t uni1=RNG_UNIF(0, maxcum), uni2=RNG_UNIF(0, maxcum);
-      long int type1, type2;
-      igraph_vector_binsearch(&cumdist, uni1, &type1);
-      igraph_vector_binsearch(&cumdist, uni2, &type2);
+      long int type1=VECTOR(nodetypes)[node1];
+      long int type2=VECTOR(nodetypes)[node2];
 /*    printf("unif: %f, %f, types: %li, %li\n", uni1, uni2, type1, type2); */
       if (RNG_UNIF01() < MATRIX(*pref_matrix, type1, type2)) {
 	IGRAPH_CHECK(igraph_vector_push_back(&edges, node1));
@@ -690,8 +700,9 @@ int igraph_callaway_traits_game (igraph_t *graph, igraph_integer_t nodes,
 
   RNG_END();
 
+  igraph_vector_destroy(&nodetypes);
   igraph_vector_destroy(&cumdist);
-  IGRAPH_FINALLY_CLEAN(1);
+  IGRAPH_FINALLY_CLEAN(2);
   IGRAPH_CHECK(igraph_create(graph, &edges, nodes, directed));
   igraph_vector_destroy(&edges);
   IGRAPH_FINALLY_CLEAN(1);
@@ -736,10 +747,12 @@ int igraph_establishment_game(igraph_t *graph, igraph_integer_t nodes,
   igraph_vector_t cumdist;
   igraph_vector_t potneis;
   igraph_real_t maxcum;
+  igraph_vector_t nodetypes;
   
   IGRAPH_VECTOR_INIT_FINALLY(&edges, 0);
   IGRAPH_VECTOR_INIT_FINALLY(&cumdist, types+1);
   IGRAPH_VECTOR_INIT_FINALLY(&potneis, k);
+  IGRAPH_VECTOR_INIT_FINALLY(&nodetypes, nodes);
   
   VECTOR(cumdist)[0]=0;
   for (i=0; i<types; i++) {
@@ -748,14 +761,19 @@ int igraph_establishment_game(igraph_t *graph, igraph_integer_t nodes,
   maxcum=igraph_vector_tail(&cumdist);
 
   RNG_BEGIN();
-  
-  for (i=k; i<nodes; i++) {
+
+  for (i=0; i<nodes; i++) {
+    igraph_real_t uni=RNG_UNIF(0, maxcum);
+    long int type;
+    igraph_vector_binsearch(&cumdist, uni, &type);
+    VECTOR(nodetypes)[i]=type;
+  }
+
+  for (i=k; i<nodes; i++) {    
+    long int type1=VECTOR(nodetypes)[i];
     igraph_random_sample(&potneis, 0, i-1, k);
     for (j=0; j<k; j++) {
-      long int type1, type2;
-      igraph_real_t uni1=RNG_UNIF(0, maxcum), uni2=RNG_UNIF(0, maxcum);
-      igraph_vector_binsearch(&cumdist, uni1, &type1);
-      igraph_vector_binsearch(&cumdist, uni2, &type2);
+      long int type2=VECTOR(nodetypes)[ (long int)VECTOR(potneis)[j] ];
       if (RNG_UNIF01() < MATRIX(*pref_matrix, type1, type2)) {
 	IGRAPH_CHECK(igraph_vector_push_back(&edges, i));
 	IGRAPH_CHECK(igraph_vector_push_back(&edges, VECTOR(potneis)[j]));
@@ -765,9 +783,10 @@ int igraph_establishment_game(igraph_t *graph, igraph_integer_t nodes,
   
   RNG_END();
   
+  igraph_vector_destroy(&nodetypes);
   igraph_vector_destroy(&potneis);
   igraph_vector_destroy(&cumdist);
-  IGRAPH_FINALLY_CLEAN(2);
+  IGRAPH_FINALLY_CLEAN(3);
   IGRAPH_CHECK(igraph_create(graph, &edges, nodes, directed));
   igraph_vector_destroy(&edges);
   IGRAPH_FINALLY_CLEAN(1);
@@ -901,7 +920,7 @@ int igraph_nonlinear_barabasi_game(igraph_t *graph, igraph_integer_t n,
   igraph_vector_destroy(&degree);
   IGRAPH_FINALLY_CLEAN(2);
 
-  IGRAPH_CHECK(igraph_create(graph, &edges, 0, directed));
+  IGRAPH_CHECK(igraph_create(graph, &edges, n, directed));
   igraph_vector_destroy(&edges);
   IGRAPH_FINALLY_CLEAN(1);
 
@@ -1043,7 +1062,7 @@ int igraph_recent_degree_game(igraph_t *graph, igraph_integer_t n,
   igraph_vector_destroy(&degree);
   IGRAPH_FINALLY_CLEAN(3);
 
-  IGRAPH_CHECK(igraph_create(graph, &edges, 0, directed));
+  IGRAPH_CHECK(igraph_create(graph, &edges, n, directed));
   igraph_vector_destroy(&edges);
   IGRAPH_FINALLY_CLEAN(1);
 
@@ -1192,7 +1211,7 @@ int igraph_barabasi_aging_game(igraph_t *graph,
       long int shnode=i-binwidth*k;
       long int deg=VECTOR(degree)[shnode];
       long int age=(i-shnode)/binwidth;
-      igraph_real_t old=igraph_psumtree_get(&sumtree, shnode);
+      /* igraph_real_t old=igraph_psumtree_get(&sumtree, shnode); */
       igraph_psumtree_update(&sumtree, shnode,
 			     (deg_coef*pow(deg, pa_exp)+zero_deg_appeal) * 
 			     (age_coef*pow(age+2, aging_exp)+zero_age_appeal));
@@ -1205,7 +1224,7 @@ int igraph_barabasi_aging_game(igraph_t *graph,
   igraph_psumtree_destroy(&sumtree);
   IGRAPH_FINALLY_CLEAN(2);
 
-  IGRAPH_CHECK(igraph_create(graph, &edges, 0, directed));
+  IGRAPH_CHECK(igraph_create(graph, &edges, nodes, directed));
   igraph_vector_destroy(&edges);
   IGRAPH_FINALLY_CLEAN(1);
   
@@ -1378,11 +1397,304 @@ int igraph_recent_degree_aging_game(igraph_t *graph,
   igraph_psumtree_destroy(&sumtree);
   IGRAPH_FINALLY_CLEAN(3);
 
-  IGRAPH_CHECK(igraph_create(graph, &edges, 0, directed));
+  IGRAPH_CHECK(igraph_create(graph, &edges, nodes, directed));
   igraph_vector_destroy(&edges);
   IGRAPH_FINALLY_CLEAN(1);
   
   return 0;
 }
+
+/**
+ * \function igraph_grg_game
+ * \brief Generating geometric random graphs.
+ *
+ * A geometric random graph is created by dropping points (=vertices)
+ * randomly to the unit square and then connecting all those pairs
+ * which are less than \c radius apart in Euclidean norm.
+ * 
+ * </para><para>
+ * Original code contributed by Keith Briggs, thanks Keith.
+ * \param graph Pointer to an uninitialized graph object,
+ * \param nodes The number of vertices in the graph.
+ * \param radius The radius within which the vertices will be connected.
+ * \param torus Logical constant, if true periodic boundary conditions
+ *        will be used, ie. the vertices are assumed to be on a torus 
+ *        instead of a square.
+ * \return Error code.
+ * 
+ * Time complexity: TODO, less than O(|V|^2+|E|).
+ */
 				    
-				   
+int igraph_grg_game(igraph_t *graph, igraph_integer_t nodes,
+		    igraph_real_t radius, igraph_bool_t torus) {
+  
+  long int i;
+  igraph_vector_t x, y, edges;
+  igraph_real_t r2=radius*radius;
+  
+  IGRAPH_VECTOR_INIT_FINALLY(&edges, 0);
+  IGRAPH_CHECK(igraph_vector_reserve(&edges, nodes));
+  IGRAPH_VECTOR_INIT_FINALLY(&x, nodes);
+  IGRAPH_VECTOR_INIT_FINALLY(&y, nodes);
+  
+  RNG_BEGIN();
+  
+  for (i=0; i<nodes; i++) {
+    VECTOR(x)[i]=RNG_UNIF01();
+    VECTOR(y)[i]=RNG_UNIF01();
+  }
+  
+  RNG_END();
+
+  igraph_vector_sort(&x);
+
+  if (!torus) {
+    for (i=0; i<nodes; i++) {
+      igraph_real_t x1=VECTOR(x)[i];
+      igraph_real_t y1=VECTOR(y)[i];
+      long int j=i+1;
+      igraph_real_t dx, dy;
+      while ( j<nodes && (dx=VECTOR(x)[j] - x1) < radius) {
+	dy=VECTOR(y)[j]-y1;
+	if (dx*dx+dy*dy < r2) {
+	  IGRAPH_CHECK(igraph_vector_push_back(&edges, i));
+	  IGRAPH_CHECK(igraph_vector_push_back(&edges, j));
+	}
+	j++;
+      }
+    }
+  } else { 
+    for (i=0; i<nodes; i++) {
+      igraph_real_t x1=VECTOR(x)[i];
+      igraph_real_t y1=VECTOR(y)[i];
+      long int j=i+1;
+      igraph_real_t dx, dy;
+      while ( j<nodes && (dx=VECTOR(x)[j] - x1) < radius) {
+	dy=fabs(VECTOR(y)[j]-y1);
+	if (dx > 0.5) {
+	  dx=1-dx;
+	}
+	if (dy > 0.5) {
+	  dy=1-dy;
+	}
+	if (dx*dx+dy*dy < r2) {
+	  IGRAPH_CHECK(igraph_vector_push_back(&edges, i));
+	  IGRAPH_CHECK(igraph_vector_push_back(&edges, j));
+	}
+	j++;
+      }
+      if (j==nodes) {
+	j=0;      
+	while (j<i && (dx=1-x1+VECTOR(x)[j]) < radius && 
+	       x1-VECTOR(x)[j]>=radius) {
+	  dy=fabs(VECTOR(y)[j]-y1);
+	  if (dy > 0.5) {
+	    dy=1-dy;
+	  }
+	  if (dx*dx+dy*dy < r2) {
+	    IGRAPH_CHECK(igraph_vector_push_back(&edges, i));
+	    IGRAPH_CHECK(igraph_vector_push_back(&edges, j));
+	  }
+	  j++;
+	}
+      }
+    }
+  }
+  
+  igraph_vector_destroy(&y);
+  igraph_vector_destroy(&x);
+  IGRAPH_FINALLY_CLEAN(2);
+
+  IGRAPH_CHECK(igraph_create(graph, &edges, nodes, IGRAPH_UNDIRECTED));
+  igraph_vector_destroy(&edges);
+  IGRAPH_FINALLY_CLEAN(1);
+  
+  return 0;
+}
+  
+/**
+ * \function igraph_preference_game
+ * \brief Generates a graph with vertex types and connection preferences 
+ * 
+ * </para><para>
+ * This is practically the nongrowing variant of \c igraph_establishment_game .
+ * A given number of vertices are generated. Every vertex is assigned to a
+ * vertex type according to the given type probabilities. Finally, every
+ * vertex pair is evaluated and an edge is created between them with a
+ * probability depending on the types of the vertices involved.
+ * 
+ * \param graph Pointer to an uninitialized graph.
+ * \param nodes The number of vertices in the graph.
+ * \param types The number of vertex types.
+ * \param type_dist Vector giving the distribution of vertex types.
+ * \param pref_matrix Matrix giving the connection probabilities for
+ * different vertex types.
+ * \param directed Logical, whether to generate a directed graph. If undirected
+ *   graphs are requested, only the lower left triangle of the preference
+ *   matrix is considered.
+ * \param loops Logical, whether loop edges are allowed.
+ * \return Error code.
+ *
+ * Added in version 0.3.</para><para>
+ *
+ * Time complexity: O(|V|^2), |V| is the number of vertices.
+ *
+ * \sa igraph_establishment_game()
+ */
+
+int igraph_preference_game(igraph_t *graph, igraph_integer_t nodes,
+			   igraph_integer_t types,
+			   igraph_vector_t *type_dist,
+			   igraph_matrix_t *pref_matrix,
+			   igraph_bool_t directed,
+			   igraph_bool_t loops) {
+  
+  long int i, j;
+  igraph_vector_t edges;
+  igraph_vector_t cumdist;
+  igraph_vector_t nodetypes;
+  igraph_real_t maxcum;
+  
+  IGRAPH_VECTOR_INIT_FINALLY(&edges, 0);
+  IGRAPH_VECTOR_INIT_FINALLY(&nodetypes, nodes);
+  IGRAPH_VECTOR_INIT_FINALLY(&cumdist, types+1);
+
+  VECTOR(cumdist)[0]=0;
+  for (i=0; i<types; i++) {
+    VECTOR(cumdist)[i+1] = VECTOR(cumdist)[i]+VECTOR(*type_dist)[i];
+  }
+  maxcum=igraph_vector_tail(&cumdist);
+
+  RNG_BEGIN();
+
+  for (i=0; i<nodes; i++) {
+    long int type1;
+    igraph_real_t uni1=RNG_UNIF(0, maxcum);
+    igraph_vector_binsearch(&cumdist, uni1, &type1);
+    VECTOR(nodetypes)[i] = type1;
+  }
+
+  igraph_vector_destroy(&cumdist);
+  IGRAPH_FINALLY_CLEAN(1);
+
+  for (i=0; i<nodes; i++) {
+    for (j=(directed?0:i); j<nodes; j++) {
+      long type1, type2;
+
+      if (!loops && i == j) continue;
+
+      type1=VECTOR(nodetypes)[i];
+      type2=VECTOR(nodetypes)[j];
+
+      if (RNG_UNIF01() < MATRIX(*pref_matrix, type1, type2)) {
+	IGRAPH_CHECK(igraph_vector_push_back(&edges, i));
+	IGRAPH_CHECK(igraph_vector_push_back(&edges, j));
+      }
+    }
+  }
+  
+  RNG_END();
+  
+  igraph_vector_destroy(&nodetypes);
+  IGRAPH_FINALLY_CLEAN(1);
+
+  IGRAPH_CHECK(igraph_create(graph, &edges, nodes, directed));
+  igraph_vector_destroy(&edges);
+  IGRAPH_FINALLY_CLEAN(1);
+  return 0;
+}
+/**
+ * \function igraph_asymmetric_preference_game
+ * \brief Generates a graph with asymmetric vertex types and connection preferences 
+ * 
+ * </para><para>
+ * This is the asymmetric variant of \ref igraph_preference_game() .
+ * A given number of vertices are generated. Every vertex is assigned to an
+ * "incoming" and an "outgoing" vertex type according to the given joint
+ * type probabilities. Finally, every vertex pair is evaluated and a
+ * directed edge is created between them with a probability depending on the
+ * "outgoing" type of the source vertex and the "incoming" type of the target
+ * vertex.
+ * 
+ * \param graph Pointer to an uninitialized graph.
+ * \param nodes The number of vertices in the graph.
+ * \param types The number of vertex types.
+ * \param type_dist_matrix Matrix giving the joint distribution of vertex types.
+ * \param pref_matrix Matrix giving the connection probabilities for
+ * different vertex types.
+ * \param loops Logical, whether loop edges are allowed.
+ * \return Error code.
+ *
+ * Added in version 0.3.</para><para>
+ *
+ * Time complexity: O(|V|^2), |V| is the number of vertices.
+ *
+ * \sa \ref igraph_preference_game()
+ */
+
+int igraph_asymmetric_preference_game(igraph_t *graph, igraph_integer_t nodes,
+				      igraph_integer_t types,
+				      igraph_matrix_t *type_dist_matrix,
+				      igraph_matrix_t *pref_matrix,
+				      igraph_bool_t loops) {
+  
+  long int i, j, k;
+  igraph_vector_t edges;
+  igraph_vector_t cumdist;
+  igraph_vector_t nodetypes_in;
+  igraph_vector_t nodetypes_out;
+  igraph_real_t maxcum;
+  
+  IGRAPH_VECTOR_INIT_FINALLY(&edges, 0);
+  IGRAPH_VECTOR_INIT_FINALLY(&nodetypes_in, nodes);
+  IGRAPH_VECTOR_INIT_FINALLY(&nodetypes_out, nodes);
+  IGRAPH_VECTOR_INIT_FINALLY(&cumdist, types*types+1);
+
+  VECTOR(cumdist)[0]=0;
+  for (i=0, k=0; i<types; i++) {
+    for (j=0; j<types; j++, k++) {
+      VECTOR(cumdist)[k+1] = VECTOR(cumdist)[k]+MATRIX(*type_dist_matrix, i, j);
+    }
+  }
+  maxcum=igraph_vector_tail(&cumdist);
+
+  RNG_BEGIN();
+
+  for (i=0; i<nodes; i++) {
+    long int type1;
+    igraph_real_t uni1=RNG_UNIF(0, maxcum);
+    igraph_vector_binsearch(&cumdist, uni1, &type1);
+    VECTOR(nodetypes_in)[i] = type1 / (int)types;
+    VECTOR(nodetypes_out)[i] = type1 % (int)types;
+  }
+
+  igraph_vector_destroy(&cumdist);
+  IGRAPH_FINALLY_CLEAN(1);
+
+  for (i=0; i<nodes; i++) {
+    for (j=0; j<nodes; j++) {
+      long type1, type2;
+
+      if (!loops && i == j) continue;
+
+      type1=VECTOR(nodetypes_out)[i];
+      type2=VECTOR(nodetypes_in)[j];
+
+      if (RNG_UNIF01() < MATRIX(*pref_matrix, type1, type2)) {
+	IGRAPH_CHECK(igraph_vector_push_back(&edges, i));
+	IGRAPH_CHECK(igraph_vector_push_back(&edges, j));
+      }
+    }
+  }
+  
+  RNG_END();
+  
+  igraph_vector_destroy(&nodetypes_out);
+  igraph_vector_destroy(&nodetypes_in);
+  IGRAPH_FINALLY_CLEAN(2);
+
+  IGRAPH_CHECK(igraph_create(graph, &edges, nodes, 1));
+  igraph_vector_destroy(&edges);
+  IGRAPH_FINALLY_CLEAN(1);
+  return 0;
+}
